program RPG;

  { Simple scrolling map demo to demonstrate overlapping tiles }

  uses
    Crt,
    RPGGfx;  { generated by Tile Studio }

  const
    W = Tiles1_TileWidth;
    H = Tiles1_TileHeight;
    Overlap = Tiles1_Overlap;
    TileCount = Tiles1_TileCount;
    MapW = Tiles1_Map1_MapWidth;
    MapH = Tiles1_Map1_MapHeight;

  const
    XScroll = 80;
    YScroll = 50;

  const
    MAX_SPEED = 2;

  const
    SCREEN_WIDTH = 320;
    SCREEN_HEIGHT = 200;

  type
    ByteArrayPtr = ^ByteArray;
    ByteArray = array[0..$FFFE] of Byte;

  type
    WordArrayPtr = ^WordArray;
    WordArray = array[0..$7FFE] of Word;

  procedure Refresh; assembler;
  asm
        mov     dx, 3DAh
  @1:   in      al, dx
        test    al, 8
        jnz     @1
  @2:   in      al, dx
        test    al, 8
        jz      @2
  end;

  procedure SetScreenMode (NewMode: Byte);
  begin
    asm
        push    bp
        xor     ah, ah
        mov     al, NewMode
        int     10h
        pop     bp
    end;
  end;

  procedure SetPalette (Color, Red, Green, Blue: Byte);
  begin
    asm
          cli
          mov     dx, 03C8h
          mov     al, Color
          out     dx, al
          inc     dx
          mov     al, Red
          out     dx, al
          mov     al, Green
          out     dx, al
          mov     al, Blue
          out     dx, al
          sti
    end;
  end;

  procedure ClipDrawImage (X, Y, W, H: Integer; var Bitmap);
    var
      XSkip, YSkip: Integer;
  begin
    XSkip := 0;
    YSkip := 0;

    if (X + W < 0) or (X >= SCREEN_WIDTH) then
      Exit;
    if X < 0 then
    begin
      XSkip := -X;
      Inc (X, XSkip);
      Dec (W, XSkip);
      asm
        mov ax, XSkip
        add word ptr [Bitmap], ax
      end;
    end
    else
      if X >= (SCREEN_WIDTH) - W then
      begin
        XSkip := W;
        W := (SCREEN_WIDTH) - X;
        Dec (XSkip, W);
      end;
    if W <= 0 then
      Exit;

    if (Y <= -H) or (Y >= SCREEN_WIDTH) then
      Exit;
    if Y < 0 then
    begin
      YSkip := -Y;
      Inc (Y, YSkip);
      Dec (H, YSkip);
      YSkip := YSkip * (XSkip + W);
    end
    else
      if Y >= SCREEN_HEIGHT - H then
        H := SCREEN_HEIGHT - Y;

    if H <= 0 then
      Exit;
    asm
        push    ds
        mov     bx, Y
        mov     ax, SCREEN_WIDTH
        mul     bx
        add     ax, X
        mov     di, ax
        mov     es, SegA000
        mov     dx, H
        lds     si, Bitmap
        add     si, YSkip
        mov     cx, W
    @@1:
        push    cx
    @@2:
        lodsb
        or      al, al
        jz      @@3
        mov     es:[di], al
    @@3:
        inc     di
        dec     cx
        jnz     @@2

        pop     cx
        add     si, XSkip
        add     di, SCREEN_WIDTH
        sub     di, cx
        dec     dx
        jnz     @@1

        pop     ds
    end;
  end;

  function CheckBounds (var X, Y, XV, YV: Integer): Byte;
  { check for bounds and get map code }
    var
      X1, Y1, X2, Y2, NewX1, NewY1, NewX2, NewY2: Integer;
      wap: WordArrayPtr;
      Bnd: Word;
      MapX1, MapY1, NewMapX1, NewMapY1: Integer;
      MapX2, MapY2, NewMapX2, NewMapY2: Integer;
      MidMapX, MidMapY: Integer;
      i: Integer;
  begin
    wap := @Tiles1_Map1_Bounds;
    X1 := X;  X2 := X + W - 1;
    Y1 := Y;  Y2 := Y + (H - Overlap) - 1;
    NewX1 := X1 + XV;  NewX2 := X2 + XV;
    NewY1 := Y1 + YV;  NewY2 := Y2 + YV;
    MapX1 := X1 div W;  MapX2 := X2 div W;
    MapY1 := Y1 div (H - Overlap);  MapY2 := Y2 div (H - Overlap);
    NewMapX1 := NewX1 div W;  NewMapX2 := NewX2 div W;
    NewMapY1 := NewY1 div (H - Overlap);  NewMapY2 := NewY2 div (H - Overlap);
    MidMapX := (X + W div 2 + XV div 2) div W;
    MidMapY := (Y + (H - Overlap) div 2 + YV div 2) div (H - Overlap);

    if NewMapX1 < MapX1 then  { Left }
      for i := NewMapY1 to NewMapY2 do
      begin
        Bnd := wap^[NewMapX1 + MapW * i];
        if Bnd and 8 <> 0 then
        begin
          X := MidMapX * W;
          XV := 0;
        end;
      end;

    if NewMapX2 > MapX2 then  { Right }
      for i := NewMapY1 to NewMapY2 do
      begin
        Bnd := wap^[NewMapX2 + MapW * i];
        if Bnd and 2 <> 0 then
        begin
          X := MidMapX * W;
          XV := 0;
        end;
      end;

    if NewMapY1 < MapY1 then  { Upper }
      for i := NewMapX1 to NewMapX2 do
      begin
        Bnd := wap^[i + MapW * NewMapY1];
        if Bnd and 4 <> 0 then
        begin
          Y := MidMapY * (H - Overlap);
          YV := 0;
        end;
      end;

    if NewMapY2 > MapY2 then  { Lower }
      for i := NewMapX1 to NewMapX2 do
      begin
        Bnd := wap^[i + MapW * NewMapY2];
        if Bnd and 1 <> 0 then
        begin
          Y := MidMapY * (H - Overlap);
          YV := 0;
        end;
      end;

    CheckBounds := wap^[MidMapX + MapW * MidMapY] shr 8;  { return map code }
  end;


  var
    bap, bapMap: ByteArrayPtr;
    XView, YView: Integer;
    i, j, k: Integer;
    x, y: Integer;
    Quit: Boolean;
    Skip: array[0..TileCount - 1] of Integer;
    xp, yp: Integer;
    xv, yv: Integer;
    MainCharTile: Integer;
    Ground: Integer;

begin
  bapMap := @Tiles1_Objects;
  MainCharTile := bapMap^[Tiles1_Objects_MapWidth] - 1;

  for k := 0 to TileCount - 1 do
  begin
    bap := ByteArrayPtr (LongInt (@Tiles1_Tiles) + k * (W * H));
    i := 0;
    while bap^[i] = 0 do
      Inc (i);
    Skip[k] := i div W;
  end;

  SetScreenMode ($13);

  bap := @Palette1;
  for i := 0 to 255 do
    SetPalette (i, bap^[3 * i + 0], bap^[3 * i + 1], bap^[3 * i + 2]);

  XView := 0;
  YView := 0;
  xv := 0;
  yv := 0;
  xp := SCREEN_WIDTH div 2;
  yp := SCREEN_HEIGHT div 2;

  bapMap := @Tiles1_Map1;
  repeat
    for j := 0 to MapH - 1 do
    begin
      y := j * (H - Overlap);
      for i := 0 to MapW - 1 do
      begin
        k := bapMap^[j * MapW + i] - 1;
        x := i * W;
        begin
          bap := ByteArrayPtr (LongInt (@Tiles1_Tiles) + k * (W * H) + Skip[k] * W);
          ClipDrawImage (x - XView, y - YView - Overlap + Skip[k], W, H - Skip[k], bap^);
        end;
      end;
      if (yp <= y) and (yp > y - (H - Overlap)) then
      begin
        k := MainCharTile;
        bap := ByteArrayPtr (LongInt (@Tiles1_Tiles) + k * (W * H) + Skip[k] * W);
        ClipDrawImage (xp - XView, yp - YView - Overlap + Skip[k], W, H - Skip[k], bap^);
      end;
    end;


    if KeyPressed then
    begin
      case ReadKey of
        #0: case ReadKey of
              'H': if yv > -MAX_SPEED then Dec (yv);
              'K': if xv > -MAX_SPEED then Dec (xv);
              'P': if yv < MAX_SPEED then Inc (yv);
              'M': if xv < MAX_SPEED then Inc (xv);
            end;
        #27: Quit := TRUE;
      end;
    end;

    Ground := CheckBounds (xp, yp, xv, yv);
    Inc (xp, xv);
    Inc (yp, yv);

    if Ground = 1 then  { on the road: faster }
    begin
      Ground := CheckBounds (xp, yp, xv, yv);
      Inc (xp, xv);
      Inc (yp, yv);
    end;


    if yp < 0 then
    begin
      yp := 0;
      yv := 0;
    end
    else
      if yp > MapH * (H - Overlap) - (H - Overlap) then
      begin
        yp := MapH * (H - Overlap) - (H - Overlap);
        yv := 0;
      end;

    if xp < 0 then
    begin
      xp := 0;
      xv := 0;
    end
    else
      if xp > MapW * W - W then
      begin
        xp := MapW * W - W;
        xv := 0;
      end;

    { scroll along with main character }
    if yp - YScroll < YView then
      YView := yp - YScroll;
    if yp + (H - Overlap) + YScroll > YView + SCREEN_HEIGHT then
      YView := yp + (H - Overlap) + YScroll - SCREEN_HEIGHT;

    if xp - XScroll < XView then
      XView := xp - XScroll;
    if xp + W + XScroll > XView + SCREEN_WIDTH then
      XView := xp + W + XScroll - SCREEN_WIDTH;


    if YView < 0 then
      YView := 0
    else
      if YView > MapH * (H - Overlap) - SCREEN_HEIGHT then
        YView := MapH * (H - Overlap) - SCREEN_HEIGHT;

    if XView < 0 then
      XView := 0
    else
      if XView > MapW * W - SCREEN_WIDTH then
        XView := MapW * W - SCREEN_WIDTH;

    Refresh;

  until Quit;

  SetScreenMode (3);
end.
